@@ -12,19 +12,19 @@ class Decider(object):
         self.bucketer = bucketer.Bucketer()
         self.logger = logger
 
-    def decide(self, experiment, user_id):
+    def decide(self, experiment, user):
         if isinstance(experiment, entities.CompletedExperiment):
             return Decision(None, experiment.winnerVariationKey, enums.DecisionType.FORCED)
 
         if isinstance(experiment, entities.RunningExperiment):
-            return self.__decide(experiment, user_id)
+            return self.__decide(experiment, user)
 
-    def __decide(self, experiment, user_id):
-        variation = experiment.userOverrides.get(user_id)
+    def __decide(self, experiment, user):
+        variation = experiment.userOverrides.get(user.id)
         if variation:
             return Decision(None, variation.key, enums.DecisionType.FORCED)
 
-        slot = self.bucketer.bucketing(experiment.bucket, user_id)
+        slot = self.bucketer.bucketing(experiment.bucket, user.id)
         if not slot:
             return None
 
