@@ -2,7 +2,7 @@ from . import decider
 from . import exceptions as hackle_exceptions
 from . import logger as _logging
 from .commons import validator, enums
-from .event import event_processor, event
+from .event import event_processor, event as event_entity
 from .event.event_dispatcher import EventDispatcher
 from .workspace_fetcher import WorkspaceFetcher
 
@@ -41,7 +41,7 @@ class Client(object):
     def __exit__(self):
         self.close()
 
-    def variation(self, experiment_key, user_id, default_variation='A'):
+    def variation(self, experiment_key, user, default_variation='A'):
         """
         Decide the variation to expose to the user for experiment.
 
@@ -51,15 +51,21 @@ class Client(object):
             - The user is not allocated to the experiment
             - The decided variation has been dropped
 
-        :param experiment_key: the unique key of the experiment.
-        :param user_id: the identifier of your customer (e.g. user_email, account_id, etc.) MUST NOT be null.
-        :param default_variation: the default variation of the experiment. MUST NOT be null.
+        :param int experiment_key: the unique key of the experiment.
+        :param hackle.model.User user: the identifier of your customer (e.g. user_email, account_id, etc.) MUST NOT be
+         null.
+        :param str default_variation: the default variation of the experiment. MUST NOT be null.
         :return: the decided variation for the user, or the default variation.
         """
         if not validator.is_non_zero_and_empty_int(experiment_key):
             self.logger.error('Experiment Key Is Not Empty : {}'.format(experiment_key))
             return default_variation
 
+        if not validator.is_valid_user(user):
+            self.logger.error('User is not valid. user\'s type must be hackle.model.User and user.id\'s type must be '
+                              'string_types : {}'.format(user))
+            return default_variation
+
         config = self.workspace_fetcher.get_config()
         if not config:
             self.logger.error('Invalid Workspace. Hackle instance is not valid. {}'.format('variation'))
@@ -71,7 +77,7 @@ class Client(object):
             self.logger.info('Experiment Key "%s" is invalid. Not Allocated.' % experiment_key)
             return default_variation
 
-        decision = self.decider.decide(experiment, user_id)
+        decision = self.decider.decide(experiment, user)
         if not decision:
             return default_variation
 
@@ -79,22 +85,28 @@ class Client(object):
             return decision.variation
         elif decision.type == enums.DecisionType.NATURAL:
             self.event_processor.process(
-                event.ExposureEvent(user_id, decision.experiment.id, decision.experiment.key, decision.variation.id,
-                                    decision.variation.key))
+                event_entity.ExposureEvent(user, decision.experiment.id, decision.experiment.key, decision.variation.id,
+                                           decision.variation.key))
             return decision.variation.key
         return default_variation
 
-    def track(self, event_key, user_id, value=None):
+    def track(self, event, user):
         """
         Records the event performed by the user with additional numeric value.
 
-        :param event_key: the unique key of the event. MUST NOT be null.
-        :param user_id: the identifier of user that performed the event. MUST NOT be null.
-        :param value: additional numeric value of the event (e.g. purchase_amount, api_latency, etc.)
+        :param hackle.model.Event event: the unique key of the event. MUST NOT be null.
+        :param user: the identifier of user that performed the event. MUST NOT be null.
         :return:
         """
-        if not validator.is_non_empty_string(event_key):
-            self.logger.error('Event Key Is Not Empty : {}'.format(event_key))
+        if not validator.is_valid_event(event):
+            self.logger.error('Event is not valid. Event must be hackle.model.event and event.id\'s type must be '
+                              'string_types. value\'s type must be numeric. '
+                              ': {}'.format(event))
+            return
+
+        if not validator.is_valid_user(user):
+            self.logger.error('User is not valid. user\'s type must be hackle.model.User and user.id\'s type must be '
+                              'string_types : {}'.format(user))
             return
 
         config = self.workspace_fetcher.get_config()
@@ -102,9 +114,21 @@ class Client(object):
             self.logger.error('Invalid Workspace. Hackle instance is not valid. {}'.format('event'))
             return
 
-        event_type = config.get_event(event_key)
+        event_type = config.get_event(event.key)
         self.event_processor.process(
-            event.TrackEvent(user_id, event_type.id, event_type.key, value)
+            event_entity.TrackEvent(user, event_type, event)
         )
 
         return
+
+    def user(self, id, properties=None):
+        if not validator.is_non_empty_string(id):
+            self.logger.error('id Is Not Empty : {}'.format(id))
+            return
+
+        if properties and validator.is_valid_user(properties):
+            self.logger.error('properties is not valid : {}'.format(id))
+            return
+
+        # TODO
+        return
